package App::Glacier::Command::Periodic;
use strict;
use warnings;
use App::Glacier::Core;
use parent qw(App::Glacier::Command);
use Carp;
use Data::Dumper;
use File::Basename;

=head1 NAME

glacier periodic - periodic cronjob for Glacier

=head1 SYNOPSIS

B<glacier periodic>

=head1 DESCRIPTION

Scans pending glacier jobs. For each job, checks its current status and if
it the job is completed, retrieves the result. Typical usage is in the
crontab.

=cut

sub run {
    my $self = shift;

    my $db = $self->jobdb();
    $db->foreach(sub {
	my ($key, $descr) = @_;
	my $vault = $descr->{VaultARN};
	$vault =~ s{.*:vaults/}{};

	return if $descr->{Completed};
	
	$self->debug(2, "$descr->{JobId} $descr->{Action} $vault");
	if ($descr->{StatusCode} eq 'Failed') {
	    $self->debug(1, "deleting failed $key $vault " .
			     ($descr->{JobDescription} || $descr->{Action}) .
			 $descr->{JobId});
	    $db->delete($key) unless $self->dry_run;
	}

	my $res = $self->glacier_eval('describe_job',
				      $vault,
				      $descr->{JobId});
	if ($self->lasterr) {
	    if ($self->lasterr('code') == 404) {
		$self->debug(1, "deleting expired $key $vault " .
			     ($descr->{JobDescription} || $descr->{Action}) .
			     $descr->{JobId});
		$db->delete($key) unless $self->dry_run;
 	    } else {
		$self->error("can't describe job $descr->{JobId}: ",
			     $self->last_error_message);
	    }
	} elsif (ref($res) ne 'HASH') {
	    croak "describe_job returned wrong datatype (".ref($res).") for \"$descr->{JobId}\"";
	} else {
	    $self->debug(2, $res->{StatusCode});
	    if ($res->{Completed} && $res->{StatusCode} eq 'Succeeded') {
		$self->debug(1, "$descr->{JobId}: processing $descr->{Action} for $vault");
		return if $self->dry_run;
		if ($res->{Action} eq 'InventoryRetrieval') {
		    require App::Glacier::Command::Sync;
		    my $sync = clone App::Glacier::Command::Sync($self);
	            $sync->sync($vault);
		} elsif ($res->{Action} eq 'ArchiveRetrieval') {
		    my $job = App::Glacier::Job->fromdb($self, $vault,
						        $key, $res);
		    my $localname = $self->archive_cache_filename($vault,
							     $res->{ArchiveId});
		    $self->touchdir(dirname($localname));

		    require App::Glacier::Command::Get;
		    my $get = clone App::Glacier::Command::Get($self);
		    $get->download($job, $localname);
	        }
	    }
	    $db->store($key, $res);
	}
		 });
}

1;

     
